@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2004, 2007  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004, 2007, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
 # Copyright (C) 2000, 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
@@ -15,12 +15,28 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: tests.sh,v 1.14.206.5 2007/08/28 07:19:09 tbox Exp $
+# $Id$
 
 SYSTEMTESTTOP=..
 . $SYSTEMTESTTOP/conf.sh
 
 status=0
+echo "I:waiting for nameserver to load"
+for i in 0 1 2 3 4 5 6 7 8 9
+do
+	ret=0
+	for zone in . example1 e.example1 example2 10.10.10.in-addr.arpa \
+	    ip6.int ip6.arpa
+	do
+		$DIG +tcp -p 5300 @10.53.0.1 soa $zone > dig.out
+		grep "status: NOERROR" dig.out > /dev/null || ret=1
+		grep "ANSWER: 1," dig.out > /dev/null || ret=1
+	done
+	test $ret = 0 && break
+	sleep 1
+done
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
 
 echo "I:using resolv.conf"
 ret=0
@@ -32,10 +48,11 @@ status=`expr $status + $ret`
 
 $PERL $SYSTEMTESTTOP/stop.pl . lwresd1
 
-$PERL $SYSTEMTESTTOP/start.pl . lwresd1 -- "-m record -c lwresd.conf -d 99 -g"
+$PERL $SYSTEMTESTTOP/start.pl . lwresd1 -- "-m record,size,mctx -c lwresd.conf -d 99 -g"
 
 echo "I:using lwresd.conf"
 ret=0
+sleep 1 # allow lwresd to finish starting.
 ./lwtest || ret=1
 if [ $ret != 0 ]; then
 	echo "I:failed"
